BorderedMorph subclass: #ScratchNoteSelector
	instanceVariableNames: 'done selectedVal labelMorph notePlayer '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Scratch-Sound'!

!ScratchNoteSelector methodsFor: 'initialization' stamp: 'MJM 3/29/2011 19:00'!
buildKeyboard: nOctaves baseOctave: baseOctave keyWidth: whiteW
	"Build keyboard for the given number of octaves starting with the given octave."

	| whiteH blackW blackH octaveStart octavePt nWhite nBlack noteR key xOffset |
	self removeAllMorphs.
	self extent: 5@5.  "adjusted later"
	whiteH _ (3.3 * whiteW) rounded.
	blackW _ (0.57 * whiteW) rounded.
	blackW even ifTrue: [blackW _ blackW - 1].
	blackH _ (0.6 * whiteH) rounded.
	0 to: nOctaves do: [:i |
		octaveStart _ 12 * (baseOctave + i).
		octavePt _ self innerBounds topLeft + (((7 * whiteW * i) - 1) @ -1).

		i < nOctaves
			ifTrue: [nWhite _ 7. nBlack _ 5]
			ifFalse: [nWhite _ 1. nBlack _ 0].  "only one 'C' key in top octave"

		1 to: nWhite do: [:j | 
			noteR _ (octavePt + (((j - 1) * whiteW) @ 0)) extent: (whiteW + 1) @ whiteH.
			key _ ScratchNoteSelectorKeyMorph newBounds: noteR.
			key midiKey: octaveStart + (#(0 2 4 5 7 9 11) at: j).
			self addMorph: key].

		1 to: nBlack do: [:j |
			xOffset _ (whiteW * (#(1 2 4 5 6) at: j)) - (blackW // 2).
			noteR _ (octavePt + (xOffset @ 0)) extent: blackW @ blackH.
			key _ ScratchNoteSelectorKeyMorph newBounds: noteR.
			key midiKey: octaveStart + (#(1 3 6 8 10) at: j).
			self addMorph: key]].

	self extent: (self fullBounds extent + borderWidth - 1).
! !

!ScratchNoteSelector methodsFor: 'initialization' stamp: 'MJM 3/30/2011 00:41'!
initialize

	super initialize.
	selectedVal _ nil.  "if nothing is ever selected, then don't return anything"
	notePlayer _ nil.
	self buildKeyboard: 2 baseOctave: 4 keyWidth: 13.
	self setUpNoteDisplay.
! !

!ScratchNoteSelector methodsFor: 'initialization' stamp: 'jm 10/22/2007 23:17'!
setNotePlayer: thePlayer

	notePlayer _ thePlayer.
! !

!ScratchNoteSelector methodsFor: 'initialization' stamp: 'MJM 3/29/2011 18:54'!
setUpNoteDisplay

	| height rectMorph |
	height _ 20.

	"set up the rect morph which holds the note text"
	rectMorph _ BorderedMorph new
		color: Color white;
		position: self bottomLeft - (0@2);
		extent: (self width @ height).
	self extent: self fullBounds extent.
	self addMorph: rectMorph.

	labelMorph _ StringMorph new.
	labelMorph position: rectMorph position + (0@4).
	rectMorph addMorph: labelMorph.
	self extent: self fullBounds extent.

	self updateNoteDisplay: 60.
! !


!ScratchNoteSelector methodsFor: 'interaction' stamp: 'jm 10/23/2007 00:19'!
startUp
	"Waits for the user to click a value or to click outside, then returns the selected note or nil."

	| w result |
	self openInWorld.
	w _ self world.
	Sensor waitNoButton.  "start with mouse up"
	w doOneCycle.
	w activeHand newMouseFocus: self.
	selectedVal _ nil.
	done _ false.
	[done] whileFalse: [
		(w activeHand hasMouseFocus: self) ifFalse: [done _ true].
		w doOneCycle].

	result _ selectedVal.
	self shutDown.
	^ result
! !


!ScratchNoteSelector methodsFor: 'event handling' stamp: 'jm 10/22/2007 23:31'!
handlesMouseDown: evt

	^ true
! !

!ScratchNoteSelector methodsFor: 'event handling' stamp: 'jm 10/26/2007 08:40'!
mouseMove: evt

	| keyMorph k |
	evt anyButtonPressed ifFalse: [^ self].
	keyMorph _ submorphs
		detect: [:m |
			(m containsPoint: evt cursorPoint) and:
			[m isKindOf: ScratchNoteSelectorKeyMorph]]
		ifNone: [nil].
	keyMorph
		ifNil: [self turnOffNote]
		ifNotNil: [
			k _ keyMorph midiKey.
			k = selectedVal ifTrue: [^ self].
			self turnOffNote.
			keyMorph highlight: true.
			self turnOnNote: k.
			selectedVal _ k].
! !


!ScratchNoteSelector methodsFor: 'stepping' stamp: 'jm 10/26/2007 08:47'!
step

	| keyMorph |
	Sensor anyButtonPressed ifTrue: [^ self].
	keyMorph _ submorphs
		detect: [:m |
			(m containsPoint: Sensor cursorPoint) and:
			[m isKindOf: ScratchNoteSelectorKeyMorph]]
		ifNone: [^ self].

	self updateNoteDisplay: keyMorph midiKey.
! !

!ScratchNoteSelector methodsFor: 'stepping' stamp: 'jm 10/26/2007 08:49'!
stepTime

	^ 100

! !


!ScratchNoteSelector methodsFor: 'private' stamp: 'NB 10/22/2007 00:55'!
shutDown
	"closes everything up, and destroys the morph"
	self turnOffNote.
	self delete.! !

!ScratchNoteSelector methodsFor: 'private' stamp: 'jm 10/26/2007 08:39'!
turnOffNote

	submorphs do: [:m | (m isKindOf: ScratchNoteSelectorKeyMorph) ifTrue: [m highlight: false]].
	notePlayer ifNotNil: [notePlayer noteOff].
	selectedVal _ nil.
! !

!ScratchNoteSelector methodsFor: 'private' stamp: 'jm 10/22/2007 23:32'!
turnOnNote: midiKey

	selectedVal _ midiKey.
	self updateNoteDisplay: midiKey.
	notePlayer ifNotNil: [notePlayer noteOn: midiKey].
! !

!ScratchNoteSelector methodsFor: 'private' stamp: 'jm 10/23/2007 00:17'!
updateNoteDisplay: noteNum

	| s |
	s _ #('C' 'C#' 'D' 'Eb' 'E' 'F' 'F#' 'G' 'G#' 'A' 'Bb' 'B') at: ((noteNum rem: 12) + 1).
	labelMorph
		contents: s, ' (' , noteNum asString, ')';
		centerInOwner.
! !
